import type { ThemedToken } from '@shikijs/types'
import type { SyntaxHighlighterProps } from '../types/shiki'
import { useEffect, useState } from 'react'
import { createHighlighter } from '../syntax/highlighter'

function useSyntaxHighlighter({ text, language, languageId, theme }: SyntaxHighlighterProps) {
  const [tokens, setTokens] = useState<ThemedToken[][]>([])
  const [error, setError] = useState<string>()

  useEffect(() => {
    let isMounted = true
    const highlightSyntax = async () => {
      if (!text) {
        setTokens([])
        return
      }

      try {
        const highlighter = await createHighlighter({ langs: [language], themes: [theme] })
        const lang = languageId || (typeof language === 'string' ? language : (language as any).name)
        const result = highlighter.codeToTokens(text, { lang, theme })

        if (isMounted && result.tokens) {
          setTokens(result.tokens)
        }
      }
      catch (err) {
        console.error('[Highlighter] Error:', err)
        if (isMounted) {
          setError(err instanceof Error ? err.message : 'Syntax highlighting failed')
        }
      }
    }

    highlightSyntax()
    return () => {
      isMounted = false
    }
  }, [text, language, languageId, theme])

  return { tokens, error }
}

export default useSyntaxHighlighter
